@@ -38,7 +38,9 @@ def get_devices():
         Device("Oculus Rift (DK1)", "2833", "0001"),
         Device("Oculus Rift (DK2)", "2833", "0021"),
         Device("Oculus Rift (DK2)", "2833", "2021"),
+        Device("Oculus Rift (DK2 Sensor)", "2833", "0201"),
         Device("Oculus Rift (CV1)", "2833", "0031"),
+        Device("Oculus Rift (CV1 Sensor)", "2833", "0211"),
         Device("Oculus Rift S", "2833", "0051"),
         Device("Samsung GearVR (Gen1)", "04e8", "a500"),
 
